# -*- coding: utf-8 -*-
"""Module contains method that will be replaced by the plugin.

:author: Pawel Chomicki
"""

import os
import re


def pytest_runtest_logstart(self, nodeid, location):
    """Signal the start of running a single test item.

    Hook has to be disabled because additional information may break output
    formatting.
    """
    pass


def pytest_collection_modifyitems(session, config, items):
    def get_module_name(f):
        return f.listchain()[1].name

    def get_nodeid(f):
        return "::".join(f.nodeid.split('::')[:-1])

    items.sort(key=get_nodeid)
    items.sort(key=get_module_name)
    return items


def get_report_scopes(report):
    """
    Returns a list of the report's nested scopes, excluding the module.

    >>> report = lambda s: s
    >>> report.nodeid = (
        "specs/user.py::describe_a_user::"
        "describe_email_address::cannot_be_hotmail"
    )
    >>> get_report_scopes(report)
    ['describe_a_user', 'describe_email_address']
    """
    return [i for i in report.nodeid.split('::')[1:-1] if i != '()']


def pytest_runtest_logreport(self, report):
    """
    Process a test setup/call/teardown report relating to the respective phase
    of executing a test.

    Hook changed to define SPECIFICATION like output format. This hook will
    overwrite also VERBOSE option.
    """
    self.previous_scopes = getattr(self, 'previous_scopes', [])
    self.current_scopes = get_report_scopes(report)
    indent = self.config.getini('spec_indent')

    res = self.config.hook.pytest_report_teststatus(report=report, config=self.config)
    cat, letter, word = res
    self.stats.setdefault(cat, []).append(report)
    if not letter and not word:
        return
    if not _is_nodeid_has_test(report.nodeid):
        return
    test_path = _get_test_path(report.nodeid, self.config.getini('spec_header_format'))
    if test_path != self.currentfspath:
        self.currentfspath = test_path
        _print_description(self)

    scope_ind = 0
    for msg in self.current_scopes:
        if msg not in self.previous_scopes:
            msg = [indent * scope_ind + prettify_description(msg)]
            msg = "\n".join(msg)
            if msg:
                _print_description(self, msg)
        scope_ind += 1
    self.previous_scopes = self.current_scopes

    if not isinstance(word, tuple):
        test_name = _get_test_name(report.nodeid)
        markup, test_status = _format_results(report, self.config)
        depth = len(self.current_scopes)
        _print_test_result(self, test_name, test_status, markup, depth)


def _is_nodeid_has_test(nodeid):
    if len(nodeid.split("::")) >= 2:
        return True
    return False


def prettify(string):
    return _capitalize_first_letter(
        _replace_underscores(
            _remove_test_container_prefix(
                _remove_file_extension(string))))


def prettify_test(string):
    return prettify(_remove_test_prefix(string))


def prettify_description(string):
    return prettify(_append_colon(_remove_test_container_prefix(string)))


def _get_test_path(nodeid, header):
    levels = nodeid.split("::")

    module_path = levels[0]
    module_name = os.path.split(levels[0])[1]

    if len(levels) > 2:
        class_name = levels[1]
        test_case = prettify(class_name)
    else:
        class_name = ''
        test_case = prettify(module_name)

    return header.format(
        path=levels[0],
        module_name=module_name,
        module_path=module_path,
        class_name=class_name,
        test_case=test_case
    )


def _print_description(self, msg=None):
    if msg is None:
        msg = self.currentfspath
    if hasattr(self, '_first_triggered'):
        self._tw.line()
    self._tw.line()
    self._tw.write(msg)
    self._first_triggered = True


def _remove_test_container_prefix(nodeid):
    return re.sub("^(Test)|(describe)", "", nodeid)


def _remove_file_extension(nodeid):
    return os.path.splitext(nodeid)[0]


def _remove_module_name(nodeid):
    return nodeid.rsplit("::", 1)[1]


def _remove_test_prefix(nodeid):
    return re.sub("^test_+", "", nodeid)


def _replace_underscores(nodeid):
    return nodeid.replace("__", " ").strip().replace("_", " ").strip()


def _capitalize_first_letter(s):
    return s[:1].capitalize() + s[1:]


def _append_colon(string):
    return "{}:".format(string)


def _get_test_name(nodeid):
    test_name = prettify_test(_remove_module_name(nodeid))
    if test_name[:1] == ' ':
        test_name_parts = test_name.split('  ')
        if len(test_name_parts) == 1:
            return test_name.strip().capitalize()
        return 'The ({0}) {1}'.format(test_name_parts[0][1:].replace(' ', '_'), test_name_parts[1])
    return test_name


def _format_results(report, config):
    success_indicator = config.getini('spec_success_indicator')
    failure_indicator = config.getini('spec_failure_indicator')
    skipped_indicator = config.getini('spec_skipped_indicator')
    if report.passed:
        return {'green': True}, success_indicator
    elif report.failed:
        return {'red': True}, failure_indicator
    elif report.skipped:
        return {'yellow': True}, skipped_indicator


def _print_test_result(self, test_name, test_status, markup, depth):
    indent = self.config.getini('spec_indent')
    self._tw.line()
    self._tw.write(
        indent * depth + self.config.getini('spec_test_format').format(
            result=test_status, name=test_name
        ), **markup
    )



# -*- coding: utf-8 -*-
"""Module contains command line option definition and logic needed to enable new formatting.

:author: Pawel Chomicki
"""
from .replacer import logstart_replacer, report_replacer, modifyitems_replacer


def pytest_addoption(parser):
    group = parser.getgroup('general')
    group.addoption(
        '--spec',
        action='store_true',
        dest='spec',
        help='Print test result in specification format'
    )

    # register config options
    parser.addini(
        'spec_header_format',
        default='{module_path}:',
        help='The format of the test headers when using the spec plugin'
    )
    parser.addini(
        'spec_test_format',
        default='{result} {name}',
        help='The format of the test results when using the spec plugin'
    )
    parser.addini(
        'spec_success_indicator',
        default='✓',
        help='The indicator displayed when a test passes'
    )
    parser.addini(
        'spec_failure_indicator',
        default='✗',
        help='The indicator displayed when a test fails'
    )
    parser.addini(
        'spec_skipped_indicator',
        default='?',
        help='The indicator displayed when a test is skipped'
    )
    parser.addini(
        'spec_indent',
        default='  ',
        help='The string used for indentation in the spec output'
    )


def pytest_configure(config):
    if getattr(config.option, 'spec', 0) and not getattr(config.option, 'quiet', 0) and not getattr(config.option, 'verbose', 0):
        import six
        import _pytest
        _pytest.terminal.TerminalReporter.pytest_runtest_logstart = logstart_replacer
        _pytest.terminal.TerminalReporter.pytest_runtest_logreport = report_replacer
        _pytest.terminal.TerminalReporter.pytest_collection_modifyitems = modifyitems_replacer
        six.moves.reload_module(_pytest)



# -*- coding: utf-8 -*-
"""Module contains method for replace operation.

Additional method are necessary because self is not yet defined and module
doesn't have access to it.

:author: Pawel Chomicki
"""
from .patch import (
    pytest_runtest_logstart,
    pytest_runtest_logreport,
    pytest_collection_modifyitems
)


def logstart_replacer(self, nodeid, location):
    def wrapper():
        return pytest_runtest_logstart(self, nodeid, location)
    return wrapper()


def report_replacer(self, report):
    def wrapper():
        return pytest_runtest_logreport(self, report)
    return wrapper()


def modifyitems_replacer(session, config, items):
    def wrapper():
        return pytest_collection_modifyitems(session, config, items)
    return wrapper()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pytest_spec']

package_data = \
{'': ['*']}

install_requires = \
['six']

entry_points = \
{'pytest11': ['pytest_spec = pytest_spec.plugin']}

setup_kwargs = {
    'name': 'pytest-spec',
    'version': '3.0.5',
    'description': 'pytest plugin to display test execution output like a SPECIFICATION',
    'long_description': 'pytest-spec\n===========\npytest plugin to display test execution output like a SPECIFICATION.\n\n\nAvailable features\n==================\n* Format output to look like specification.\n* Group tests by classes and files\n* Failed, passed and skipped are marked and colored.\n* Remove test\\_ and underscores for every test.\n* Supports function based, class based test.\n* Supports describe like tests.\n\n\nOutput example\n==============\n\n.. image:: https://github.com/pchomik/pytest-spec/raw/master/docs/output.png\n\n\nConfiguration\n=============\n\n``spec_header_format``\n----------------------\n\nYou can configure the format of the test headers by specifying a `format string <https://docs.python.org/2/library/string.html#format-string-syntax>`_ in your `ini-file <http://doc.pytest.org/en/latest/customize.html#inifiles>`_:\n\n::\n\n    [pytest]\n    spec_header_format = {module_path}:\n\nIn addition to the ``{path}`` and ``{class_name}`` replacement fields, there is also ``{test_case}`` that holds a more human readable name.\n\n``spec_test_format``\n--------------------\n\nYou can configure the format of the test results by specifying a `format string <https://docs.python.org/2/library/string.html#format-string-syntax>`_ in your `ini-file <http://doc.pytest.org/en/latest/customize.html#inifiles>`_:\n\n::\n\n    [pytest]\n    spec_test_format = {result} {name}\n\n``spec_success_indicator``\n--------------------------\n\nYou can configure the indicator displayed when test passed.\n\n::\n\n    [pytest]\n    spec_success_indicator = ✓\n\n``spec_failure_indicator``\n--------------------------\n\nYou can configure the indicator displated when test failed.\n\n::\n\n    [pytest]\n    spec_failure_indicator = ✗\n\n``spec_skipped_indicator``\n--------------------------\n\nYou can configure the indicator displated when test is skipped.\n\n::\n\n    [pytest]\n    spec_skipped_indicator = ?\n\n``spec_indent``\n---------------\n\n::\n\n    [pytest]\n    spec_indent = "   "\n\nContinuous Integration\n======================\n.. image:: https://github.com/pchomik/pytest-spec/workflows/test/badge.svg\n     :target: https://github.com/pchomik/pytest-spec/actions\n\nDownload\n========\nAll versions of library are available on official `pypi server <https://pypi.org/project/pytest-spec/#history>`_.\n\nInstall\n=======\n::\n\n    pip install pytest-spec\n\nContribution\n============\nPlease feel free to present your idea by code example (pull request) or reported issues.\n\nContributors\n============\n* @0x64746b\n* @lucasmarshall\n* @amcgregor\n* @jhermann\n* @frenzymadness\n* @chrischambers\n* @maxalbert\n* @jayvdb\n\nLicense\n=======\npytest-spec - pytest plugin to display test execution output like a SPECIFICATION.\n\nCopyright (C) 2014-2019 Pawel Chomicki\n\nThis program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.\n\nThis program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.\n',
    'author': 'Pawel Chomicki',
    'author_email': 'pawel.chomicki@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/pchomik/pytest-spec',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
}


setup(**setup_kwargs)
